@@ -2,8 +2,10 @@ from abc import ABC, abstractmethod
 from dataclasses import dataclass
 from datetime import datetime
 from typing import List, Any, Optional, Iterable
+from warnings import warn
 
 from pandas import DataFrame
+from plotly import graph_objects as go
 
 from entropylab.api.data_writer import PlotGenerator
 
@@ -84,7 +86,8 @@ class ResultRecord:
 @dataclass
 class PlotRecord:
     """
-    A single plot information and plotting instructions that was saved during the experiment
+    A single plot information and plotting instructions that was saved during the
+    experiment
     """
 
     experiment_id: int
@@ -95,6 +98,18 @@ class PlotRecord:
     story: Optional[str] = None
 
 
+@dataclass
+class FigureRecord:
+    """
+    A single plotly figure that was saved during the experiment
+    """
+
+    experiment_id: int
+    id: int
+    figure: go.Figure
+    time: datetime
+
+
 @dataclass
 class NodeResults:
     """
@@ -116,19 +131,24 @@ class DataReader(ABC):
     def __init__(self):
         super().__init__()
 
-    def get_last_experiments(self, count: int) -> DataFrame:
+    def get_last_experiments(self, count: int, success: bool = None) -> DataFrame:
         """
             read n number of last experiments to a pandas dataframe
         :param count: number of experiments
+        :param success: Optional filter for the success property.
         """
-        return self.get_experiments_range(0, count)
+        return self.get_experiments_range(0, count, success)
 
     @abstractmethod
-    def get_experiments_range(self, starting_from_index: int, count: int) -> DataFrame:
+    def get_experiments_range(
+        self, starting_from_index: int, count: int, success: bool = None
+    ) -> DataFrame:
         """
             read a range of experiments to a pandas dataframe
         :param starting_from_index: experiment index to start from (sorted desc by time)
         :param count: number of experiments
+        :param success: Optional filter for the success property.
+        :return: A DataFrame containing one row per Experiment
         """
         pass
 
@@ -149,7 +169,8 @@ class DataReader(ABC):
         success: Optional[bool] = None,
     ) -> Iterable[ExperimentRecord]:
         """
-            get multiple experiments records according to any combination of parameters filters
+            get multiple experiments records according to any combination of parameters
+            filters
         :param label: experiment label to filter by
         :param start_after: experiments start after specific time
         :param end_after: experiments ended after specific time
@@ -181,7 +202,8 @@ class DataReader(ABC):
         stage: Optional[int] = None,
     ) -> Iterable[MetadataRecord]:
         """
-            get multiple metadata records according to any combination of parameters filters
+            get multiple metadata records according to any combination of parameters
+            filters
 
         :param experiment_id: metadata from specific experiment
         :param label: metadata label to filter by
@@ -205,19 +227,33 @@ class DataReader(ABC):
         """
         pass
 
+    # noinspection PyTypeChecker
     @abstractmethod
     def get_plots(self, experiment_id: int) -> List[PlotRecord]:
         """
         returns a list of all plots saved in the requested experiment
         """
+        warn(
+            "This method will soon be deprecated. Please use get_figures() instead",
+            PendingDeprecationWarning,
+            stacklevel=2,
+        )
         pass
 
     @abstractmethod
-    def get_nodes_id_by_label(
+    def get_figures(self, experiment_id: int) -> List[FigureRecord]:
+        """
+        returns a list of all figures saved in the requested experiment
+        """
+        pass
+
+    @abstractmethod
+    def get_node_stage_ids_by_label(
         self, label: str, experiment_id: Optional[int] = None
     ) -> List[int]:
         """
-            returns a list of all nodes with the given label
+            returns a list of the stage_id property values of all nodes with the
+            given label
         :param label: results label
         :param experiment_id: optional to specify an experiment id, so
                 only results from this experiment will be returned
@@ -239,18 +275,18 @@ class DataReader(ABC):
                 only results from this experiment will be returned
         :param result_label: optional to specify a label of the results
         """
-        nodes = self.get_nodes_id_by_label(node_label, experiment_id)
-        if not nodes:
+        node_stage_ids = self.get_node_stage_ids_by_label(node_label, experiment_id)
+        if not node_stage_ids:
             raise KeyError(f"node {node_label} not found")
         nodes_results = []
-        for node_id in nodes:
+        for stage_id in node_stage_ids:
             nodes_results.append(
                 NodeResults(
-                    node_id,
+                    stage_id,
                     self.get_results(
                         experiment_id,
                         label=result_label,
-                        stage=node_id,
+                        stage=stage_id,
                     ),
                 )
             )
@@ -304,4 +340,15 @@ class ExperimentReader:
         """
         returns a list of plot records that were saved for current experiment
         """
+        warn(
+            "This method will soon be deprecated. Please use get_plots() instead",
+            PendingDeprecationWarning,
+            stacklevel=2,
+        )
         return self._data_reader.get_plots(self._experiment_id)
+
+    def get_figures(self) -> List[FigureRecord]:
+        """
+        returns a list of plotly figures that were saved for current experiment
+        """
+        return self._data_reader.get_figures(self._experiment_id)
